package javax.swing.text;

import java.util.Enumeration;
import java.util.Stack;

public class ElementIterator implements Cloneable {
   private Element root;
   private Stack elementStack = null;

   public ElementIterator(Document var1) {
      this.root = var1.getDefaultRootElement();
   }

   public ElementIterator(Element var1) {
      this.root = var1;
   }

   public synchronized Object clone() {
      try {
         ElementIterator var1 = new ElementIterator(this.root);
         if (this.elementStack != null) {
            var1.elementStack = new Stack();

            for(int var2 = 0; var2 < this.elementStack.size(); ++var2) {
               StackItem var3 = (StackItem)this.elementStack.elementAt(var2);
               StackItem var4 = (StackItem)var3.clone();
               var1.elementStack.push(var4);
            }
         }

         return var1;
      } catch (CloneNotSupportedException var5) {
         throw new InternalError();
      }
   }

   public Element first() {
      if (this.root == null) {
         return null;
      } else {
         this.elementStack = new Stack();
         if (this.root.getElementCount() != 0) {
            this.elementStack.push(new StackItem(this, this.root, (1)null));
         }

         return this.root;
      }
   }

   public int depth() {
      return this.elementStack == null ? 0 : this.elementStack.size();
   }

   public Element current() {
      if (this.elementStack == null) {
         return this.first();
      } else if (!this.elementStack.empty()) {
         StackItem var1 = (StackItem)this.elementStack.peek();
         Element var2 = javax.swing.text.ElementIterator.StackItem.access$100(var1);
         int var3 = javax.swing.text.ElementIterator.StackItem.access$200(var1);
         return var3 == -1 ? var2 : var2.getElement(var3);
      } else {
         return null;
      }
   }

   public Element next() {
      if (this.elementStack == null) {
         return this.first();
      } else if (this.elementStack.isEmpty()) {
         return null;
      } else {
         StackItem var1 = (StackItem)this.elementStack.peek();
         Element var2 = javax.swing.text.ElementIterator.StackItem.access$100(var1);
         int var3 = javax.swing.text.ElementIterator.StackItem.access$200(var1);
         if (var3 + 1 < var2.getElementCount()) {
            Element var5 = var2.getElement(var3 + 1);
            if (var5.isLeaf()) {
               javax.swing.text.ElementIterator.StackItem.access$300(var1);
            } else {
               this.elementStack.push(new StackItem(this, var5, (1)null));
            }

            return var5;
         } else {
            this.elementStack.pop();
            if (!this.elementStack.isEmpty()) {
               StackItem var4 = (StackItem)this.elementStack.peek();
               javax.swing.text.ElementIterator.StackItem.access$300(var4);
               return this.next();
            } else {
               return null;
            }
         }
      }
   }

   public Element previous() {
      int var1;
      if (this.elementStack != null && (var1 = this.elementStack.size()) != 0) {
         StackItem var2 = (StackItem)this.elementStack.peek();
         Element var3 = javax.swing.text.ElementIterator.StackItem.access$100(var2);
         int var4 = javax.swing.text.ElementIterator.StackItem.access$200(var2);
         if (var4 > 0) {
            --var4;
            return this.getDeepestLeaf(var3.getElement(var4));
         } else if (var4 == 0) {
            return var3;
         } else if (var4 == -1) {
            if (var1 == 1) {
               return null;
            } else {
               Object var5 = this.elementStack.pop();
               var2 = (StackItem)this.elementStack.peek();
               this.elementStack.push(var5);
               var3 = javax.swing.text.ElementIterator.StackItem.access$100(var2);
               var4 = javax.swing.text.ElementIterator.StackItem.access$200(var2);
               return var4 == -1 ? var3 : this.getDeepestLeaf(var3.getElement(var4));
            }
         } else {
            return null;
         }
      } else {
         return null;
      }
   }

   private Element getDeepestLeaf(Element var1) {
      if (var1.isLeaf()) {
         return var1;
      } else {
         int var2 = var1.getElementCount();
         return var2 == 0 ? var1 : this.getDeepestLeaf(var1.getElement(var2 - 1));
      }
   }

   private void dumpTree() {
      Element var1;
      while((var1 = this.next()) != null) {
         System.out.println("elem: " + var1.getName());
         AttributeSet var2 = var1.getAttributes();
         String var3 = "";
         Enumeration var4 = var2.getAttributeNames();

         while(var4.hasMoreElements()) {
            Object var5 = var4.nextElement();
            Object var6 = var2.getAttribute(var5);
            if (var6 instanceof AttributeSet) {
               var3 = var3 + var5 + "=**AttributeSet** ";
            } else {
               var3 = var3 + var5 + "=" + var6 + " ";
            }
         }

         System.out.println("attributes: " + var3);
      }

   }
}
