package javax.swing.text.html;

import java.awt.Color;
import java.awt.Font;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.Reader;
import java.io.StringReader;
import java.net.URL;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.text.AttributeSet;
import javax.swing.text.Element;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.View;
import javax.swing.text.html.HTML.Attribute;

public class StyleSheet extends StyleContext {
   static final Border noBorder = new EmptyBorder(0, 0, 0, 0);
   static final int DEFAULT_FONT_SIZE = 3;
   private CSS css;
   private Hashtable selectorMapping = new Hashtable();
   private Hashtable resolvedStyles = new Hashtable();
   private Vector linkedStyleSheets;
   private URL base;
   static final SheetAttribute SPECIFICITY = new SheetAttribute("specificity", false);
   static final SheetAttribute RULE = new SheetAttribute("rule", false);

   public StyleSheet() {
      if (this.css == null) {
         this.css = new CSS();
      }

   }

   public Style getRule(HTML.Tag var1, Element var2) {
      SearchBuffer var3 = javax.swing.text.html.StyleSheet.SearchBuffer.obtainSearchBuffer();

      Style var13;
      try {
         Vector var4 = var3.getVector();

         for(Element var5 = var2; var5 != null; var5 = var5.getParentElement()) {
            var4.addElement(var5);
         }

         int var6 = var4.size();
         StringBuffer var7 = var3.getStringBuffer();

         for(int var11 = var6 - 1; var11 >= 1; --var11) {
            var2 = (Element)var4.elementAt(var11);
            AttributeSet var8 = var2.getAttributes();
            Object var10 = var8.getAttribute(StyleConstants.NameAttribute);
            String var9 = var10.toString();
            var7.append(var9);
            if (var8 != null) {
               if (var8.isDefined(Attribute.ID)) {
                  var7.append('#');
                  var7.append(var8.getAttribute(Attribute.ID));
               } else if (var8.isDefined(Attribute.CLASS)) {
                  var7.append('.');
                  var7.append(var8.getAttribute(Attribute.CLASS));
               }
            }

            var7.append(' ');
         }

         var7.append(var1.toString());
         var2 = (Element)var4.elementAt(0);
         AttributeSet var20 = var2.getAttributes();
         if (var2.isLeaf()) {
            Object var12 = var20.getAttribute(var1);
            if (var12 instanceof AttributeSet) {
               var20 = (AttributeSet)var12;
            } else {
               var20 = null;
            }
         }

         if (var20 != null) {
            if (var20.isDefined(Attribute.ID)) {
               var7.append('#');
               var7.append(var20.getAttribute(Attribute.ID));
            } else if (var20.isDefined(Attribute.CLASS)) {
               var7.append('.');
               var7.append(var20.getAttribute(Attribute.CLASS));
            }
         }

         Style var21 = this.getResolvedStyle(var7.toString(), var4, var1);
         var13 = var21;
      } finally {
         javax.swing.text.html.StyleSheet.SearchBuffer.releaseSearchBuffer(var3);
      }

      return var13;
   }

   public Style getRule(String var1) {
      var1 = this.cleanSelectorString(var1);
      if (var1 != null) {
         Style var2 = this.getResolvedStyle(var1);
         return var2;
      } else {
         return null;
      }
   }

   public void addRule(String var1) {
      if (var1 != null) {
         CssParser var2 = new CssParser(this);

         try {
            var2.parse(this.getBase(), new StringReader(var1), false, false);
         } catch (IOException var4) {
         }
      }

   }

   public AttributeSet getDeclaration(String var1) {
      if (var1 == null) {
         return SimpleAttributeSet.EMPTY;
      } else {
         CssParser var2 = new CssParser(this);
         return var2.parseDeclaration(var1);
      }
   }

   public void loadRules(Reader var1, URL var2) throws IOException {
      CssParser var3 = new CssParser(this);
      var3.parse(var2, var1, false, false);
   }

   public AttributeSet getViewAttributes(View var1) {
      return new ViewAttributeSet(this, var1);
   }

   public void removeStyle(String var1) {
      Style var2 = ((StyleContext)this).getStyle(var1);
      if (var2 != null) {
         String var3 = this.cleanSelectorString(var1);
         String[] var4 = this.getSimpleSelectors(var3);
         synchronized(this) {
            Object var6 = this.getRootSelectorMapping();

            for(int var7 = var4.length - 1; var7 >= 0; --var7) {
               var6 = this.getSelectorMapping(var6, var4[var7], true);
            }

            Style var8 = this.getMappingStyle(var6);
            if (var8 != null) {
               this.removeMappingStyle(var6);
               if (this.resolvedStyles.size() > 0) {
                  Enumeration var9 = this.resolvedStyles.elements();

                  while(var9.hasMoreElements()) {
                     ResolvedStyle var10 = (ResolvedStyle)var9.nextElement();
                     var10.removeStyle(var8);
                  }
               }
            }
         }
      }

      super.removeStyle(var1);
   }

   public void addStyleSheet(StyleSheet var1) {
      synchronized(this) {
         if (this.linkedStyleSheets == null) {
            this.linkedStyleSheets = new Vector();
         }

         if (!this.linkedStyleSheets.contains(var1)) {
            this.linkedStyleSheets.insertElementAt(var1, 0);
            this.linkStyleSheetAt(var1, 0);
         }

      }
   }

   public void removeStyleSheet(StyleSheet var1) {
      synchronized(this) {
         if (this.linkedStyleSheets != null) {
            int var3 = this.linkedStyleSheets.indexOf(var1);
            if (var3 != -1) {
               this.linkedStyleSheets.removeElementAt(var3);
               this.unlinkStyleSheet(var1, var3);
               if (var3 == 0 && this.linkedStyleSheets.size() == 0) {
                  this.linkedStyleSheets = null;
               }
            }
         }

      }
   }

   public StyleSheet[] getStyleSheets() {
      synchronized(this) {
         StyleSheet[] var1;
         if (this.linkedStyleSheets != null) {
            var1 = new StyleSheet[this.linkedStyleSheets.size()];
            this.linkedStyleSheets.copyInto(var1);
         } else {
            var1 = null;
         }

         return var1;
      }
   }

   public void importStyleSheet(URL var1) {
      try {
         InputStream var2 = var1.openStream();
         BufferedReader var3 = new BufferedReader(new InputStreamReader(var2));
         CssParser var4 = new CssParser(this);
         var4.parse(var1, var3, false, true);
         ((Reader)var3).close();
         var2.close();
      } catch (Throwable var5) {
      }

   }

   public void setBase(URL var1) {
      this.base = var1;
   }

   public URL getBase() {
      return this.base;
   }

   public void addCSSAttribute(MutableAttributeSet var1, CSS.Attribute var2, String var3) {
      this.css.addInternalCSSValue(var1, var2, var3);
   }

   public boolean addCSSAttributeFromHTML(MutableAttributeSet var1, CSS.Attribute var2, String var3) {
      Object var4 = this.css.getCssValue(var2, var3);
      if (var4 != null) {
         var1.addAttribute(var2, var4);
         return true;
      } else {
         return false;
      }
   }

   public AttributeSet translateHTMLToCSS(AttributeSet var1) {
      AttributeSet var2 = this.css.translateHTMLToCSS(var1);
      Style var3 = ((StyleContext)this).addStyle((String)null, (Style)null);
      var3.addAttributes(var2);
      return var3;
   }

   public AttributeSet addAttribute(AttributeSet var1, Object var2, Object var3) {
      if (this.css == null) {
         this.css = new CSS();
      }

      if (var2 instanceof StyleConstants) {
         Object var4 = this.css.styleConstantsValueToCSSValue((StyleConstants)var2, var3);
         if (var4 != null) {
            CSS.Attribute var5 = this.css.styleConstantsKeyToCSSKey((StyleConstants)var2);
            if (var5 != null) {
               return super.addAttribute(var1, var5, var4);
            }
         }
      }

      return super.addAttribute(var1, var2, var3);
   }

   public AttributeSet addAttributes(AttributeSet var1, AttributeSet var2) {
      return super.addAttributes(var1, this.convertAttributeSet(var2));
   }

   public AttributeSet removeAttribute(AttributeSet var1, Object var2) {
      if (var2 instanceof StyleConstants) {
         CSS.Attribute var3 = this.css.styleConstantsKeyToCSSKey((StyleConstants)var2);
         if (var3 != null) {
            return super.removeAttribute(var1, var3);
         }
      }

      return super.removeAttribute(var1, var2);
   }

   public AttributeSet removeAttributes(AttributeSet var1, Enumeration var2) {
      return super.removeAttributes(var1, var2);
   }

   public AttributeSet removeAttributes(AttributeSet var1, AttributeSet var2) {
      return super.removeAttributes(var1, this.convertAttributeSet(var2));
   }

   protected StyleContext.SmallAttributeSet createSmallAttributeSet(AttributeSet var1) {
      return new SmallConversionSet(this, var1);
   }

   protected MutableAttributeSet createLargeAttributeSet(AttributeSet var1) {
      return new LargeConversionSet(this, var1);
   }

   AttributeSet convertAttributeSet(AttributeSet var1) {
      if (!(var1 instanceof LargeConversionSet) && !(var1 instanceof SmallConversionSet)) {
         Enumeration var2 = var1.getAttributeNames();

         while(var2.hasMoreElements()) {
            Object var3 = var2.nextElement();
            if (var3 instanceof StyleConstants) {
               LargeConversionSet var4 = new LargeConversionSet(this);
               Enumeration var5 = var1.getAttributeNames();

               while(var5.hasMoreElements()) {
                  Object var6 = var5.nextElement();
                  Object var7 = null;
                  if (var6 instanceof StyleConstants) {
                     CSS.Attribute var8 = this.css.styleConstantsKeyToCSSKey((StyleConstants)var6);
                     if (var8 != null) {
                        Object var9 = var1.getAttribute(var6);
                        var7 = this.css.styleConstantsValueToCSSValue((StyleConstants)var6, var9);
                        if (var7 != null) {
                           var4.addAttribute(var8, var7);
                        }
                     }
                  }

                  if (var7 == null) {
                     var4.addAttribute(var6, var1.getAttribute(var6));
                  }
               }

               return var4;
            }
         }

         return var1;
      } else {
         return var1;
      }
   }

   public Font getFont(AttributeSet var1) {
      return this.css.getFont(this, var1, 12);
   }

   public Color getForeground(AttributeSet var1) {
      Color var2 = this.css.getColor(var1, javax.swing.text.html.CSS.Attribute.COLOR);
      return var2 == null ? Color.black : var2;
   }

   public Color getBackground(AttributeSet var1) {
      return this.css.getColor(var1, javax.swing.text.html.CSS.Attribute.BACKGROUND_COLOR);
   }

   public BoxPainter getBoxPainter(AttributeSet var1) {
      return new BoxPainter(var1, this.css, this);
   }

   public ListPainter getListPainter(AttributeSet var1) {
      return new ListPainter(var1, this);
   }

   public void setBaseFontSize(int var1) {
      this.css.setBaseFontSize(var1);
   }

   public void setBaseFontSize(String var1) {
      this.css.setBaseFontSize(var1);
   }

   public static int getIndexOfSize(float var0) {
      return CSS.getIndexOfSize(var0);
   }

   public float getPointSize(int var1) {
      return this.css.getPointSize(var1);
   }

   public float getPointSize(String var1) {
      return this.css.getPointSize(var1);
   }

   public Color stringToColor(String var1) {
      return CSS.stringToColor(var1);
   }

   ImageIcon getBackgroundImage(AttributeSet var1) {
      Object var2 = var1.getAttribute(javax.swing.text.html.CSS.Attribute.BACKGROUND_IMAGE);
      return var2 != null ? ((CSS.BackgroundImage)var2).getImage(this.getBase()) : null;
   }

   void addRule(String[] var1, AttributeSet var2, boolean var3) {
      int var4 = var1.length;
      StringBuffer var5 = new StringBuffer();
      var5.append(var1[0]);

      for(int var6 = 1; var6 < var4; ++var6) {
         var5.append(' ');
         var5.append(var1[var6]);
      }

      String var7 = var5.toString();
      Style var8 = ((StyleContext)this).getStyle(var7);
      if (var8 == null) {
         Style var9 = ((StyleContext)this).addStyle(var7, (Style)null);
         synchronized(this) {
            Object var11 = this.getRootSelectorMapping();

            for(int var12 = var4 - 1; var12 >= 0; --var12) {
               var11 = this.getSelectorMapping(var11, var1[var12], true);
            }

            var8 = this.getMappingStyle(var11);
            if (var8 == null) {
               var8 = this.createStyleForSelector(var7, var11, var9);
               this.refreshResolvedRules(var7, var1, var8, this.getSpecificity(var11));
            }
         }
      }

      if (var3) {
         var8 = this.getLinkedStyle(var8);
      }

      var8.addAttributes(var2);
   }

   private synchronized void linkStyleSheetAt(StyleSheet var1, int var2) {
      if (this.resolvedStyles.size() > 0) {
         Enumeration var3 = this.resolvedStyles.elements();

         while(var3.hasMoreElements()) {
            ResolvedStyle var4 = (ResolvedStyle)var3.nextElement();
            var4.insertExtendedStyleAt(var1.getRule(var4.getName()), var2);
         }
      }

   }

   private synchronized void unlinkStyleSheet(StyleSheet var1, int var2) {
      if (this.resolvedStyles.size() > 0) {
         Enumeration var3 = this.resolvedStyles.elements();

         while(var3.hasMoreElements()) {
            ResolvedStyle var4 = (ResolvedStyle)var3.nextElement();
            var4.removeExtendedStyleAt(var2);
         }
      }

   }

   String[] getSimpleSelectors(String var1) {
      var1 = this.cleanSelectorString(var1);
      SearchBuffer var2 = javax.swing.text.html.StyleSheet.SearchBuffer.obtainSearchBuffer();
      Vector var3 = var2.getVector();
      int var4 = 0;
      int var5 = var1.length();

      while(var4 != -1) {
         int var6 = var1.indexOf(32, var4);
         if (var6 != -1) {
            var3.addElement(var1.substring(var4, var6));
            ++var6;
            if (var6 == var5) {
               var4 = -1;
            } else {
               var4 = var6;
            }
         } else {
            var3.addElement(var1.substring(var4));
            var4 = -1;
         }
      }

      String[] var9 = new String[var3.size()];
      var3.copyInto(var9);
      javax.swing.text.html.StyleSheet.SearchBuffer.releaseSearchBuffer(var2);
      return var9;
   }

   String cleanSelectorString(String var1) {
      boolean var2 = true;
      int var3 = 0;

      for(int var4 = var1.length(); var3 < var4; ++var3) {
         switch (var1.charAt(var3)) {
            case '\t':
            case '\n':
            case '\r':
               return this._cleanSelectorString(var1);
            case ' ':
               if (var2) {
                  return this._cleanSelectorString(var1);
               }

               var2 = true;
               break;
            default:
               var2 = false;
         }
      }

      if (var2) {
         return this._cleanSelectorString(var1);
      } else {
         return var1;
      }
   }

   private String _cleanSelectorString(String var1) {
      SearchBuffer var2 = javax.swing.text.html.StyleSheet.SearchBuffer.obtainSearchBuffer();
      StringBuffer var3 = var2.getStringBuffer();
      boolean var4 = true;
      int var5 = 0;
      char[] var6 = var1.toCharArray();
      int var7 = var6.length;
      Object var8 = null;

      try {
         for(int var9 = 0; var9 < var7; ++var9) {
            switch (var6[var9]) {
               case '\t':
               case '\n':
               case '\r':
                  if (!var4) {
                     var4 = true;
                     if (var5 < var9) {
                        var3.append(var6, var5, var9 - var5);
                        var3.append(' ');
                     }
                  }

                  var5 = var9 + 1;
                  break;
               case ' ':
                  if (!var4) {
                     var4 = true;
                     if (var5 < var9) {
                        var3.append(var6, var5, 1 + var9 - var5);
                     }
                  }

                  var5 = var9 + 1;
                  break;
               default:
                  var4 = false;
            }
         }

         if (var4 && var3.length() > 0) {
            var3.setLength(var3.length() - 1);
         } else if (var5 < var7) {
            var3.append(var6, var5, var7 - var5);
         }

         var14 = var3.toString();
      } finally {
         javax.swing.text.html.StyleSheet.SearchBuffer.releaseSearchBuffer(var2);
      }

      return var14;
   }

   private Object getRootSelectorMapping() {
      return this.selectorMapping;
   }

   private synchronized Object getSelectorMapping(Object var1, String var2, boolean var3) {
      Hashtable var4 = (Hashtable)((Hashtable)var1).get(var2);
      if (var4 == null && var3) {
         var4 = new Hashtable(7);
         ((Hashtable)var1).put(var2, var4);
         int var5 = 0;
         if (var1 != null) {
            Object var6 = ((Hashtable)var1).get(SPECIFICITY);
            if (var6 != null) {
               var5 = (Integer)var6;
            }
         }

         char var8 = var2.charAt(0);
         if (var8 == '.') {
            var5 += 100;
         } else if (var8 == '#') {
            var5 += 10000;
         } else {
            ++var5;
            if (var2.indexOf(46) != -1) {
               var5 += 100;
            }

            if (var2.indexOf(35) != -1) {
               var5 += 10000;
            }
         }

         var4.put(SPECIFICITY, new Integer(var5));
      }

      return var4;
   }

   static int getSpecificity(String var0) {
      int var1 = 0;
      boolean var2 = true;
      int var3 = 0;

      for(int var4 = var0.length(); var3 < var4; ++var3) {
         switch (var0.charAt(var3)) {
            case ' ':
               var2 = true;
               break;
            case '#':
               var1 += 10000;
               break;
            case '.':
               var1 += 100;
               break;
            default:
               if (var2) {
                  var2 = false;
                  ++var1;
               }
         }
      }

      return var1;
   }

   private int getSpecificity(Object var1) {
      Object var2 = ((Hashtable)var1).get(SPECIFICITY);
      return var2 != null ? (Integer)var2 : 0;
   }

   private Style getMappingStyle(Object var1) {
      return (Style)((Hashtable)var1).get(RULE);
   }

   private void removeMappingStyle(Object var1) {
      ((Hashtable)var1).remove(RULE);
   }

   private Style getLinkedStyle(Style var1) {
      Style var2 = (Style)var1.getResolveParent();
      if (var2 == null) {
         var2 = ((StyleContext)this).addStyle((String)null, (Style)null);
         var1.setResolveParent(var2);
      }

      return var2;
   }

   private synchronized Style createStyleForSelector(String var1, Object var2, Style var3) {
      Style var4 = (Style)((Hashtable)var2).get(RULE);
      if (var4 == null) {
         var4 = var3;
         ((Hashtable)var2).put(RULE, var3);
      }

      return var4;
   }

   private synchronized Style getResolvedStyle(String var1, Vector var2, HTML.Tag var3) {
      Style var4 = (Style)this.resolvedStyles.get(var1);
      if (var4 == null) {
         var4 = this.createResolvedStyle(var1, var2, var3);
      }

      return var4;
   }

   private synchronized Style getResolvedStyle(String var1) {
      Style var2 = (Style)this.resolvedStyles.get(var1);
      if (var2 == null) {
         var2 = this.createResolvedStyle(var1);
      }

      return var2;
   }

   private void addSortedStyle(Object var1, Vector var2) {
      int var3 = var2.size();
      if (var3 > 0) {
         int var4 = this.getSpecificity(var1);

         for(int var5 = 0; var5 < var3; ++var5) {
            if (var4 >= this.getSpecificity(var2.elementAt(var5))) {
               var2.insertElementAt(var1, var5);
               return;
            }
         }
      }

      var2.addElement(var1);
   }

   private synchronized void getStyles(Object var1, Vector var2, String[] var3, String[] var4, String[] var5, int var6, int var7, Hashtable var8) {
      if (!var8.contains(var1)) {
         var8.put(var1, var1);
         Style var9 = this.getMappingStyle(var1);
         if (var9 != null) {
            this.addSortedStyle(var1, var2);
         }

         for(int var10 = var6; var10 < var7; ++var10) {
            String var11 = var3[var10];
            if (var11 != null) {
               Object var12 = this.getSelectorMapping(var1, var11, false);
               if (var12 != null) {
                  this.getStyles(var12, var2, var3, var4, var5, var10 + 1, var7, var8);
               }

               if (var5[var10] != null) {
                  String var13 = var5[var10];
                  var12 = this.getSelectorMapping(var1, var11 + "." + var13, false);
                  if (var12 != null) {
                     this.getStyles(var12, var2, var3, var4, var5, var10 + 1, var7, var8);
                  }

                  var12 = this.getSelectorMapping(var1, "." + var13, false);
                  if (var12 != null) {
                     this.getStyles(var12, var2, var3, var4, var5, var10 + 1, var7, var8);
                  }
               }

               if (var4[var10] != null) {
                  String var18 = var4[var10];
                  var12 = this.getSelectorMapping(var1, var11 + "#" + var18, false);
                  if (var12 != null) {
                     this.getStyles(var12, var2, var3, var4, var5, var10 + 1, var7, var8);
                  }

                  var12 = this.getSelectorMapping(var1, "#" + var18, false);
                  if (var12 != null) {
                     this.getStyles(var12, var2, var3, var4, var5, var10 + 1, var7, var8);
                  }
               }
            }
         }

      }
   }

   private synchronized Style createResolvedStyle(String var1, String[] var2, String[] var3, String[] var4) {
      SearchBuffer var5 = javax.swing.text.html.StyleSheet.SearchBuffer.obtainSearchBuffer();
      Vector var6 = var5.getVector();
      Hashtable var7 = var5.getHashtable();

      ResolvedStyle var18;
      try {
         Object var8 = this.getRootSelectorMapping();
         int var9 = var2.length;
         String var10 = var2[0];
         Object var11 = this.getSelectorMapping(var8, var10, false);
         if (var11 != null) {
            this.getStyles(var11, var6, var2, var3, var4, 1, var9, var7);
         }

         if (var4[0] != null) {
            String var12 = var4[0];
            var11 = this.getSelectorMapping(var8, var10 + "." + var12, false);
            if (var11 != null) {
               this.getStyles(var11, var6, var2, var3, var4, 1, var9, var7);
            }

            var11 = this.getSelectorMapping(var8, "." + var12, false);
            if (var11 != null) {
               this.getStyles(var11, var6, var2, var3, var4, 1, var9, var7);
            }
         }

         if (var3[0] != null) {
            String var27 = var3[0];
            var11 = this.getSelectorMapping(var8, var10 + "#" + var27, false);
            if (var11 != null) {
               this.getStyles(var11, var6, var2, var3, var4, 1, var9, var7);
            }

            var11 = this.getSelectorMapping(var8, "#" + var27, false);
            if (var11 != null) {
               this.getStyles(var11, var6, var2, var3, var4, 1, var9, var7);
            }
         }

         int var28 = this.linkedStyleSheets != null ? this.linkedStyleSheets.size() : 0;
         int var13 = var6.size();
         AttributeSet[] var14 = new AttributeSet[var13 + var28];

         for(int var15 = 0; var15 < var13; ++var15) {
            var14[var15] = this.getMappingStyle(var6.elementAt(var15));
         }

         for(int var16 = 0; var16 < var28; ++var16) {
            Style var17 = ((StyleSheet)this.linkedStyleSheets.elementAt(var16)).getRule(var1);
            if (var17 == null) {
               var14[var16 + var13] = SimpleAttributeSet.EMPTY;
            } else {
               var14[var16 + var13] = var17;
            }
         }

         ResolvedStyle var29 = new ResolvedStyle(var1, var14, var13);
         this.resolvedStyles.put(var1, var29);
         var18 = var29;
      } finally {
         javax.swing.text.html.StyleSheet.SearchBuffer.releaseSearchBuffer(var5);
      }

      return var18;
   }

   private Style createResolvedStyle(String var1, Vector var2, HTML.Tag var3) {
      int var4 = var2.size();
      String[] var5 = new String[var4];
      String[] var6 = new String[var4];
      String[] var7 = new String[var4];

      for(int var8 = 0; var8 < var4; ++var8) {
         Element var9 = (Element)var2.elementAt(var8);
         AttributeSet var10 = var9.getAttributes();
         if (var8 == 0 && var9.isLeaf()) {
            Object var11 = var10.getAttribute(var3);
            if (var11 instanceof AttributeSet) {
               var10 = (AttributeSet)var11;
            } else {
               var10 = null;
            }
         }

         if (var10 != null) {
            HTML.Tag var12 = (HTML.Tag)var10.getAttribute(StyleConstants.NameAttribute);
            if (var12 != null) {
               var5[var8] = var12.toString();
            } else {
               var5[var8] = null;
            }

            if (var10.isDefined(Attribute.CLASS)) {
               var7[var8] = var10.getAttribute(Attribute.CLASS).toString();
            } else {
               var7[var8] = null;
            }

            if (var10.isDefined(Attribute.ID)) {
               var6[var8] = var10.getAttribute(Attribute.ID).toString();
            } else {
               var6[var8] = null;
            }
         } else {
            var5[var8] = var6[var8] = var7[var8] = null;
         }
      }

      var5[0] = var3.toString();
      return this.createResolvedStyle(var1, var5, var6, var7);
   }

   private Style createResolvedStyle(String var1) {
      SearchBuffer var2 = javax.swing.text.html.StyleSheet.SearchBuffer.obtainSearchBuffer();
      Vector var3 = var2.getVector();

      Style var17;
      try {
         int var5 = 0;
         int var6 = 0;
         int var7 = 0;
         int var8 = 0;

         for(int var9 = var1.length(); var8 < var9; var8 = var6 + 1) {
            if (var5 == var8) {
               var5 = var1.indexOf(46, var8);
            }

            if (var7 == var8) {
               var7 = var1.indexOf(35, var8);
            }

            var6 = var1.indexOf(32, var8);
            if (var6 == -1) {
               var6 = var9;
            }

            if (var5 != -1 && var7 != -1 && var5 < var6 && var7 < var6) {
               if (var7 < var5) {
                  if (var8 == var7) {
                     var3.addElement("");
                  } else {
                     var3.addElement(var1.substring(var8, var7));
                  }

                  if (var5 + 1 < var6) {
                     var3.addElement(var1.substring(var5 + 1, var6));
                  } else {
                     var3.addElement((Object)null);
                  }

                  if (var7 + 1 == var5) {
                     var3.addElement((Object)null);
                  } else {
                     var3.addElement(var1.substring(var7 + 1, var5));
                  }
               } else if (var7 < var6) {
                  if (var8 == var5) {
                     var3.addElement("");
                  } else {
                     var3.addElement(var1.substring(var8, var5));
                  }

                  if (var5 + 1 < var7) {
                     var3.addElement(var1.substring(var5 + 1, var7));
                  } else {
                     var3.addElement((Object)null);
                  }

                  if (var7 + 1 == var6) {
                     var3.addElement((Object)null);
                  } else {
                     var3.addElement(var1.substring(var7 + 1, var6));
                  }
               }

               var5 = var7 = var6 + 1;
            } else if (var5 != -1 && var5 < var6) {
               if (var5 == var8) {
                  var3.addElement("");
               } else {
                  var3.addElement(var1.substring(var8, var5));
               }

               if (var5 + 1 == var6) {
                  var3.addElement((Object)null);
               } else {
                  var3.addElement(var1.substring(var5 + 1, var6));
               }

               var3.addElement((Object)null);
               var5 = var6 + 1;
            } else if (var7 != -1 && var7 < var6) {
               if (var7 == var8) {
                  var3.addElement("");
               } else {
                  var3.addElement(var1.substring(var8, var7));
               }

               var3.addElement((Object)null);
               if (var7 + 1 == var6) {
                  var3.addElement((Object)null);
               } else {
                  var3.addElement(var1.substring(var7 + 1, var6));
               }

               var7 = var6 + 1;
            } else {
               var3.addElement(var1.substring(var8, var6));
               var3.addElement((Object)null);
               var3.addElement((Object)null);
            }
         }

         int var10 = var3.size();
         int var11 = var10 / 3;
         String[] var12 = new String[var11];
         String[] var13 = new String[var11];
         String[] var14 = new String[var11];
         int var15 = 0;

         for(int var16 = var10 - 3; var15 < var11; var16 -= 3) {
            var12[var15] = (String)var3.elementAt(var16);
            var13[var15] = (String)var3.elementAt(var16 + 1);
            var14[var15] = (String)var3.elementAt(var16 + 2);
            ++var15;
         }

         var17 = this.createResolvedStyle(var1, var12, var13, var14);
      } finally {
         javax.swing.text.html.StyleSheet.SearchBuffer.releaseSearchBuffer(var2);
      }

      return var17;
   }

   private synchronized void refreshResolvedRules(String var1, String[] var2, Style var3, int var4) {
      if (this.resolvedStyles.size() > 0) {
         Enumeration var5 = this.resolvedStyles.elements();

         while(var5.hasMoreElements()) {
            ResolvedStyle var6 = (ResolvedStyle)var5.nextElement();
            if (var6.matches(var1)) {
               var6.insertStyle(var3, var4);
            }
         }
      }

   }

   private void readObject(ObjectInputStream var1) throws ClassNotFoundException, IOException {
      var1.defaultReadObject();
      this.updateSheetAttributes(this.selectorMapping);
   }

   private void updateSheetAttributes(Hashtable var1) {
      Enumeration var2 = var1.keys();

      while(var2.hasMoreElements()) {
         Object var3 = var2.nextElement();
         Object var4 = var1.get(var3);
         if (var3 instanceof SheetAttribute) {
            if (var3.toString().equals("specificity")) {
               var1.remove(var3);
               var1.put(SPECIFICITY, var4);
            } else if (var3.toString().equals("rule")) {
               var1.remove(var3);
               var1.put(RULE, var4);
            }
         } else if (var4 instanceof Hashtable) {
            this.updateSheetAttributes((Hashtable)var4);
         }
      }

   }

   // $FF: synthetic method
   static CSS access$000(StyleSheet var0) {
      return var0.css;
   }
}
