package javax.swing.text;

import java.awt.font.TextAttribute;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.PrintStream;
import java.io.Serializable;
import java.security.AccessController;
import java.util.Dictionary;
import java.util.EventListener;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.EventListenerList;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.event.DocumentEvent.EventType;
import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoableEdit;

public abstract class AbstractDocument implements Document, Serializable {
   private transient int numReaders;
   private transient Thread currWriter;
   private static Boolean defaultI18NProperty;
   private Dictionary documentProperties;
   protected EventListenerList listenerList;
   private Content data;
   private AttributeContext context;
   private transient BranchElement bidiRoot;
   private static final String BAD_LOCK_STATE = "document lock failure";
   protected static final String BAD_LOCATION = "document location failure";
   public static final String ParagraphElementName = "paragraph";
   public static final String ContentElementName = "content";
   public static final String SectionElementName = "section";
   public static final String BidiElementName = "bidi level";
   public static final String ElementNameAttribute = "$ename";
   static final String I18NProperty = "i18n";
   static final String AsyncLoadPriority = "load priority";
   // $FF: synthetic field
   static Class class$javax$swing$event$DocumentListener;
   // $FF: synthetic field
   static Class class$javax$swing$event$UndoableEditListener;

   protected AbstractDocument(Content var1) {
      this(var1, StyleContext.getDefaultStyleContext());
   }

   protected AbstractDocument(Content var1, AttributeContext var2) {
      this.documentProperties = null;
      this.listenerList = new EventListenerList();
      this.data = var1;
      this.context = var2;
      this.bidiRoot = new BidiRootElement(this);
      if (defaultI18NProperty == null) {
         Object var3 = AccessController.doPrivileged(new 1(this));
         if (var3 != null) {
            defaultI18NProperty = Boolean.valueOf((String)var3);
         } else {
            defaultI18NProperty = Boolean.FALSE;
         }
      }

      this.putProperty("i18n", defaultI18NProperty);
      this.writeLock();

      try {
         Element[] var8 = new Element[]{new BidiElement(this, this.bidiRoot, 0, 1, 0)};
         this.bidiRoot.replace(0, 0, var8);
      } finally {
         this.writeUnlock();
      }

   }

   public Dictionary getDocumentProperties() {
      if (this.documentProperties == null) {
         this.documentProperties = new Hashtable(2);
      }

      return this.documentProperties;
   }

   public void setDocumentProperties(Dictionary var1) {
      this.documentProperties = var1;
   }

   protected void fireInsertUpdate(DocumentEvent var1) {
      Object[] var2 = this.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == (class$javax$swing$event$DocumentListener == null ? (class$javax$swing$event$DocumentListener = class$("javax.swing.event.DocumentListener")) : class$javax$swing$event$DocumentListener)) {
            ((DocumentListener)var2[var3 + 1]).insertUpdate(var1);
         }
      }

   }

   protected void fireChangedUpdate(DocumentEvent var1) {
      Object[] var2 = this.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == (class$javax$swing$event$DocumentListener == null ? (class$javax$swing$event$DocumentListener = class$("javax.swing.event.DocumentListener")) : class$javax$swing$event$DocumentListener)) {
            ((DocumentListener)var2[var3 + 1]).changedUpdate(var1);
         }
      }

   }

   protected void fireRemoveUpdate(DocumentEvent var1) {
      Object[] var2 = this.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == (class$javax$swing$event$DocumentListener == null ? (class$javax$swing$event$DocumentListener = class$("javax.swing.event.DocumentListener")) : class$javax$swing$event$DocumentListener)) {
            ((DocumentListener)var2[var3 + 1]).removeUpdate(var1);
         }
      }

   }

   protected void fireUndoableEditUpdate(UndoableEditEvent var1) {
      Object[] var2 = this.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == (class$javax$swing$event$UndoableEditListener == null ? (class$javax$swing$event$UndoableEditListener = class$("javax.swing.event.UndoableEditListener")) : class$javax$swing$event$UndoableEditListener)) {
            ((UndoableEditListener)var2[var3 + 1]).undoableEditHappened(var1);
         }
      }

   }

   public EventListener[] getListeners(Class var1) {
      return this.listenerList.getListeners(var1);
   }

   public int getAsynchronousLoadPriority() {
      Integer var1 = (Integer)this.getProperty("load priority");
      return var1 != null ? var1 : -1;
   }

   public void setAsynchronousLoadPriority(int var1) {
      Integer var2 = var1 >= 0 ? new Integer(var1) : null;
      this.putProperty("load priority", var2);
   }

   public void render(Runnable var1) {
      this.readLock();

      try {
         var1.run();
      } finally {
         this.readUnlock();
      }

   }

   public int getLength() {
      return this.data.length() - 1;
   }

   public void addDocumentListener(DocumentListener var1) {
      this.listenerList.add(class$javax$swing$event$DocumentListener == null ? (class$javax$swing$event$DocumentListener = class$("javax.swing.event.DocumentListener")) : class$javax$swing$event$DocumentListener, var1);
   }

   public void removeDocumentListener(DocumentListener var1) {
      this.listenerList.remove(class$javax$swing$event$DocumentListener == null ? (class$javax$swing$event$DocumentListener = class$("javax.swing.event.DocumentListener")) : class$javax$swing$event$DocumentListener, var1);
   }

   public void addUndoableEditListener(UndoableEditListener var1) {
      this.listenerList.add(class$javax$swing$event$UndoableEditListener == null ? (class$javax$swing$event$UndoableEditListener = class$("javax.swing.event.UndoableEditListener")) : class$javax$swing$event$UndoableEditListener, var1);
   }

   public void removeUndoableEditListener(UndoableEditListener var1) {
      this.listenerList.remove(class$javax$swing$event$UndoableEditListener == null ? (class$javax$swing$event$UndoableEditListener = class$("javax.swing.event.UndoableEditListener")) : class$javax$swing$event$UndoableEditListener, var1);
   }

   public final Object getProperty(Object var1) {
      return this.getDocumentProperties().get(var1);
   }

   public final void putProperty(Object var1, Object var2) {
      if (var2 != null) {
         this.getDocumentProperties().put(var1, var2);
      } else {
         this.getDocumentProperties().remove(var1);
      }

      if (var1 == TextAttribute.RUN_DIRECTION && Boolean.TRUE.equals(this.getProperty("i18n"))) {
         this.writeLock();

         try {
            DefaultDocumentEvent var3 = new DefaultDocumentEvent(this, 0, this.getLength(), EventType.INSERT);
            this.updateBidi(var3);
         } finally {
            this.writeUnlock();
         }
      }

   }

   public void remove(int var1, int var2) throws BadLocationException {
      if (var2 > 0) {
         this.writeLock();

         try {
            DefaultDocumentEvent var3 = new DefaultDocumentEvent(this, var1, var2, EventType.REMOVE);
            boolean var4 = false;
            if (Utilities.is1dot2) {
               Element var5;
               for(var5 = this.getDefaultRootElement(); !var5.isLeaf(); var5 = var5.getElement(var5.getElementIndex(var1))) {
               }

               var4 = Utilities.isComposedTextElement(var5);
            }

            this.removeUpdate(var3);
            UndoableEdit var10 = this.data.remove(var1, var2);
            if (var10 != null) {
               var3.addEdit(var10);
            }

            this.postRemoveUpdate(var3);
            ((CompoundEdit)var3).end();
            this.fireRemoveUpdate(var3);
            if (var10 != null && !var4) {
               this.fireUndoableEditUpdate(new UndoableEditEvent(this, var3));
            }
         } finally {
            this.writeUnlock();
         }
      }

   }

   public void insertString(int var1, String var2, AttributeSet var3) throws BadLocationException {
      if (var2 != null && var2.length() != 0) {
         this.writeLock();

         try {
            UndoableEdit var4 = this.data.insertString(var1, var2);
            DefaultDocumentEvent var5 = new DefaultDocumentEvent(this, var1, var2.length(), EventType.INSERT);
            if (var4 != null) {
               var5.addEdit(var4);
            }

            if (this.getProperty("i18n").equals(Boolean.FALSE)) {
               Object var6 = this.getProperty(TextAttribute.RUN_DIRECTION);
               if (var6 != null && var6.equals(TextAttribute.RUN_DIRECTION_RTL)) {
                  this.putProperty("i18n", Boolean.TRUE);
               } else {
                  int var7 = var2.length();

                  for(int var8 = 0; var8 < var7; ++var8) {
                     char var9 = var2.charAt(var8);
                     if (Bidi.requiresBidi(var9)) {
                        this.putProperty("i18n", Boolean.TRUE);
                        break;
                     }
                  }
               }
            }

            this.insertUpdate(var5, var3);
            ((CompoundEdit)var5).end();
            this.fireInsertUpdate(var5);
            if (var4 != null && (var3 == null || !var3.isDefined(StyleConstants.ComposedTextAttribute))) {
               this.fireUndoableEditUpdate(new UndoableEditEvent(this, var5));
            }
         } finally {
            this.writeUnlock();
         }

      }
   }

   public String getText(int var1, int var2) throws BadLocationException {
      if (var2 < 0) {
         throw new BadLocationException("Length must be positive", var2);
      } else {
         String var3 = this.data.getString(var1, var2);
         return var3;
      }
   }

   public void getText(int var1, int var2, Segment var3) throws BadLocationException {
      if (var2 < 0) {
         throw new BadLocationException("Length must be positive", var2);
      } else {
         this.data.getChars(var1, var2, var3);
      }
   }

   public synchronized Position createPosition(int var1) throws BadLocationException {
      return this.data.createPosition(var1);
   }

   public final Position getStartPosition() {
      Position var1;
      try {
         var1 = this.createPosition(0);
      } catch (BadLocationException var3) {
         var1 = null;
      }

      return var1;
   }

   public final Position getEndPosition() {
      Position var1;
      try {
         var1 = this.createPosition(this.data.length());
      } catch (BadLocationException var3) {
         var1 = null;
      }

      return var1;
   }

   public Element[] getRootElements() {
      Element[] var1 = new Element[]{this.getDefaultRootElement(), this.getBidiRootElement()};
      return var1;
   }

   public abstract Element getDefaultRootElement();

   public Element getBidiRootElement() {
      return this.bidiRoot;
   }

   boolean isLeftToRight(int var1, int var2) {
      if (!this.getProperty("i18n").equals(Boolean.TRUE)) {
         return true;
      } else {
         Element var3 = this.getBidiRootElement();
         int var4 = var3.getElementIndex(var1);
         Element var5 = var3.getElement(var4);
         if (var5.getEndOffset() >= var2) {
            AttributeSet var6 = var5.getAttributes();
            return StyleConstants.getBidiLevel(var6) % 2 == 0;
         } else {
            return true;
         }
      }
   }

   public abstract Element getParagraphElement(int var1);

   protected final AttributeContext getAttributeContext() {
      return this.context;
   }

   protected void insertUpdate(DefaultDocumentEvent var1, AttributeSet var2) {
      if (this.getProperty("i18n").equals(Boolean.TRUE)) {
         this.updateBidi(var1);
      }

   }

   protected void removeUpdate(DefaultDocumentEvent var1) {
   }

   protected void postRemoveUpdate(DefaultDocumentEvent var1) {
      if (this.getProperty("i18n").equals(Boolean.TRUE)) {
         this.updateBidi(var1);
      }

   }

   void updateBidi(DefaultDocumentEvent var1) {
      int var2;
      int var3;
      if (javax.swing.text.AbstractDocument.DefaultDocumentEvent.access$000(var1) != EventType.INSERT && javax.swing.text.AbstractDocument.DefaultDocumentEvent.access$000(var1) != EventType.CHANGE) {
         if (javax.swing.text.AbstractDocument.DefaultDocumentEvent.access$000(var1) != EventType.REMOVE) {
            throw new Error("Internal error: unknown event type.");
         }

         Element var18 = this.getParagraphElement(var1.getOffset());
         var2 = var18.getStartOffset();
         var3 = var18.getEndOffset();
      } else {
         int var4 = var1.getOffset();
         int var5 = var4 + var1.getLength();
         var2 = this.getParagraphElement(var4).getStartOffset();
         var3 = this.getParagraphElement(var5).getEndOffset();
      }

      byte[] var19 = this.calculateBidiLevels(var2, var3);
      Vector var20 = new Vector();
      int var6 = var2;
      int var7 = 0;
      if (var2 > 0) {
         int var8 = this.bidiRoot.getElementIndex(var2 - 1);
         var7 = var8;
         Element var9 = this.bidiRoot.getElement(var8);
         int var10 = StyleConstants.getBidiLevel(var9.getAttributes());
         if (var10 == var19[0]) {
            var6 = var9.getStartOffset();
         } else if (var9.getEndOffset() > var2) {
            var20.addElement(new BidiElement(this, this.bidiRoot, var9.getStartOffset(), var2, var10));
         } else {
            var7 = var8 + 1;
         }
      }

      int var21;
      for(var21 = 0; var21 < var19.length && var19[var21] == var19[0]; ++var21) {
      }

      int var22 = var3;
      BidiElement var23 = null;
      int var11 = this.bidiRoot.getElementCount() - 1;
      if (var3 <= this.getLength()) {
         int var12 = this.bidiRoot.getElementIndex(var3);
         var11 = var12;
         Element var13 = this.bidiRoot.getElement(var12);
         int var14 = StyleConstants.getBidiLevel(var13.getAttributes());
         if (var14 == var19[var19.length - 1]) {
            var22 = var13.getEndOffset();
         } else if (var13.getStartOffset() < var3) {
            var23 = new BidiElement(this, this.bidiRoot, var3, var13.getEndOffset(), var14);
         } else {
            var11 = var12 - 1;
         }
      }

      int var24;
      for(var24 = var19.length; var24 > var21 && var19[var24 - 1] == var19[var19.length - 1]; --var24) {
      }

      if (var21 == var24 && var19[0] == var19[var19.length - 1]) {
         var20.addElement(new BidiElement(this, this.bidiRoot, var6, var22, var19[0]));
      } else {
         var20.addElement(new BidiElement(this, this.bidiRoot, var6, var21 + var2, var19[0]));

         int var27;
         for(int var25 = var21; var25 < var24; var25 = var27) {
            for(var27 = var25; var27 < var19.length && var19[var27] == var19[var25]; ++var27) {
            }

            var20.addElement(new BidiElement(this, this.bidiRoot, var2 + var25, var2 + var27, var19[var25]));
         }

         var20.addElement(new BidiElement(this, this.bidiRoot, var24 + var2, var22, var19[var19.length - 1]));
      }

      if (var23 != null) {
         var20.addElement(var23);
      }

      int var26 = 0;
      if (this.bidiRoot.getElementCount() > 0) {
         var26 = var11 - var7 + 1;
      }

      Element[] var28 = new Element[var26];

      for(int var15 = 0; var15 < var26; ++var15) {
         var28[var15] = this.bidiRoot.getElement(var7 + var15);
      }

      Element[] var16 = new Element[var20.size()];
      var20.copyInto(var16);
      ElementEdit var17 = new ElementEdit(this.bidiRoot, var7, var28, var16);
      var1.addEdit(var17);
      this.bidiRoot.replace(var7, var28.length, var16);
   }

   private byte[] calculateBidiLevels(int var1, int var2) {
      byte[] var3 = new byte[var2 - var1];
      int var4 = 0;
      Boolean var5 = null;
      Object var6 = this.getProperty(TextAttribute.RUN_DIRECTION);
      if (var6 instanceof Boolean) {
         var5 = (Boolean)var6;
      }

      Element var8;
      for(int var7 = var1; var7 < var2; var7 = var8.getEndOffset()) {
         var8 = this.getParagraphElement(var7);
         int var9 = var8.getStartOffset();
         int var10 = var8.getEndOffset();
         Boolean var11 = var5;
         var6 = var8.getAttributes().getAttribute(TextAttribute.RUN_DIRECTION);
         if (var6 instanceof Boolean) {
            var11 = (Boolean)var6;
         }

         String var12;
         try {
            var12 = this.getText(var9, var10 - var9);
         } catch (BadLocationException var15) {
            throw new Error("Internal error: " + ((Throwable)var15).toString());
         }

         Bidi var13;
         if (var11 != null) {
            boolean var14 = var11.equals(TextAttribute.RUN_DIRECTION_LTR);
            var13 = new Bidi(var12.toCharArray(), var14);
         } else {
            var13 = new Bidi(var12.toCharArray());
         }

         byte[] var17 = var13.getLevels();
         System.arraycopy(var17, 0, var3, var4, var17.length);
         var4 += var17.length;
      }

      if (var4 != var3.length) {
         throw new Error("levelsEnd assertion failed.");
      } else {
         return var3;
      }
   }

   public void dump(PrintStream var1) {
      Element var2 = this.getDefaultRootElement();
      if (var2 instanceof AbstractElement) {
         ((AbstractElement)var2).dump(var1, 0);
      }

      this.bidiRoot.dump(var1, 0);
   }

   protected final Content getContent() {
      return this.data;
   }

   protected Element createLeafElement(Element var1, AttributeSet var2, int var3, int var4) {
      return new LeafElement(this, var1, var2, var3, var4);
   }

   protected Element createBranchElement(Element var1, AttributeSet var2) {
      return new BranchElement(this, var1, var2);
   }

   protected final synchronized Thread getCurrentWriter() {
      return this.currWriter;
   }

   protected final synchronized void writeLock() {
      try {
         while(this.numReaders > 0 || this.currWriter != null) {
            if (Thread.currentThread() == this.currWriter) {
               throw new IllegalStateException("Attempt to mutate in notification");
            }

            this.wait();
         }

         this.currWriter = Thread.currentThread();
      } catch (InterruptedException var2) {
         throw new Error("Interrupted attempt to aquire write lock");
      }
   }

   protected final synchronized void writeUnlock() {
      this.currWriter = null;
      this.notifyAll();
   }

   public final synchronized void readLock() {
      try {
         while(this.currWriter != null) {
            if (this.currWriter == Thread.currentThread()) {
               return;
            }

            this.wait();
         }

         ++this.numReaders;
      } catch (InterruptedException var2) {
         throw new Error("Interrupted attempt to aquire read lock");
      }
   }

   public final synchronized void readUnlock() {
      if (this.currWriter != Thread.currentThread()) {
         if (this.numReaders <= 0) {
            throw new StateInvariantError("document lock failure");
         } else {
            --this.numReaders;
            this.notify();
         }
      }
   }

   private void readObject(ObjectInputStream var1) throws ClassNotFoundException, IOException {
      var1.defaultReadObject();
      this.listenerList = new EventListenerList();
      this.bidiRoot = new BidiRootElement(this);

      try {
         this.writeLock();
         Element[] var2 = new Element[]{new BidiElement(this, this.bidiRoot, 0, 1, 0)};
         this.bidiRoot.replace(0, 0, var2);
      } finally {
         this.writeUnlock();
      }

      var1.registerValidation(new 2(this), 0);
   }

   // $FF: synthetic method
   static Class class$(String var0) {
      try {
         return Class.forName(var0);
      } catch (ClassNotFoundException var2) {
         throw new NoClassDefFoundError(((Throwable)var2).getMessage());
      }
   }
}
