package javax.swing.text;

import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;
import java.awt.event.KeyEvent;
import java.awt.font.TextHitInfo;
import java.awt.im.InputMethodRequests;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Reader;
import java.io.Writer;
import java.security.AccessController;
import java.text.AttributedCharacterIterator;
import java.text.AttributedString;
import java.util.Hashtable;
import javax.accessibility.Accessible;
import javax.accessibility.AccessibleContext;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JViewport;
import javax.swing.KeyStroke;
import javax.swing.Scrollable;
import javax.swing.UIManager;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.plaf.TextUI;

public abstract class JTextComponent extends JComponent implements Scrollable, Accessible {
   public static final String FOCUS_ACCELERATOR_KEY = "focusAcceleratorKey";
   private Document model;
   private transient Caret caret;
   private transient Highlighter highlighter;
   private transient Keymap keymap;
   private transient MutableCaretEvent caretEvent;
   private Color caretColor;
   private Color selectionColor;
   private Color selectedTextColor;
   private Color disabledTextColor;
   private boolean editable;
   private Insets margin;
   private char focusAccelerator;
   private static ClipboardOwner defaultClipboardOwner = new ClipboardObserver();
   private static Hashtable keymapTable = null;
   private JTextComponent editor;
   private static JTextComponent focusedComponent;
   private InputMethodRequests inputMethodRequestsHandler;
   private AttributedString composedText;
   private String composedTextContent;
   private Position composedTextStart;
   private Position composedTextEnd;
   private ComposedTextCaret composedTextCaret;
   private transient Caret originalCaret;
   private boolean needToSendKeyTypedEvent;
   public static final String DEFAULT_KEYMAP = "default";
   // $FF: synthetic field
   static Class class$javax$swing$event$CaretListener;

   public JTextComponent() {
      ((Component)this).enableEvents(2056L);
      this.needToSendKeyTypedEvent = !this.isProcessInputMethodEventOverridden();
      this.caretEvent = new MutableCaretEvent(this);
      ((Component)this).addMouseListener(this.caretEvent);
      ((Component)this).addFocusListener(this.caretEvent);
      this.setEditable(true);
      ((Container)this).setLayout((LayoutManager)null);
      this.updateUI();
   }

   public TextUI getUI() {
      return (TextUI)super.ui;
   }

   public void setUI(TextUI var1) {
      super.setUI(var1);
   }

   public void updateUI() {
      this.setUI((TextUI)UIManager.getUI(this));
      ((Container)this).invalidate();
   }

   public void addCaretListener(CaretListener var1) {
      super.listenerList.add(class$javax$swing$event$CaretListener == null ? (class$javax$swing$event$CaretListener = class$("javax.swing.event.CaretListener")) : class$javax$swing$event$CaretListener, var1);
   }

   public void removeCaretListener(CaretListener var1) {
      super.listenerList.remove(class$javax$swing$event$CaretListener == null ? (class$javax$swing$event$CaretListener = class$("javax.swing.event.CaretListener")) : class$javax$swing$event$CaretListener, var1);
   }

   protected void fireCaretUpdate(CaretEvent var1) {
      Object[] var2 = super.listenerList.getListenerList();

      for(int var3 = var2.length - 2; var3 >= 0; var3 -= 2) {
         if (var2[var3] == (class$javax$swing$event$CaretListener == null ? (class$javax$swing$event$CaretListener = class$("javax.swing.event.CaretListener")) : class$javax$swing$event$CaretListener)) {
            ((CaretListener)var2[var3 + 1]).caretUpdate(var1);
         }
      }

   }

   public void setDocument(Document var1) {
      if (super.accessibleContext != null) {
         this.model.removeDocumentListener((AccessibleJTextComponent)super.accessibleContext);
      }

      Document var2 = this.model;
      this.model = var1;
      ((JComponent)this).firePropertyChange("document", var2, var1);
      ((JComponent)this).revalidate();
      ((Component)this).repaint();
      if (super.accessibleContext != null) {
         this.model.addDocumentListener((AccessibleJTextComponent)super.accessibleContext);
      }

   }

   public Document getDocument() {
      return this.model;
   }

   public Action[] getActions() {
      return this.getUI().getEditorKit(this).getActions();
   }

   public void setMargin(Insets var1) {
      Insets var2 = this.margin;
      this.margin = var1;
      ((JComponent)this).firePropertyChange("margin", var2, var1);
      ((Container)this).invalidate();
   }

   public Insets getMargin() {
      return this.margin;
   }

   public Caret getCaret() {
      return this.caret;
   }

   public void setCaret(Caret var1) {
      if (this.caret != null) {
         this.caret.removeChangeListener(this.caretEvent);
         this.caret.deinstall(this);
      }

      Caret var2 = this.caret;
      this.caret = var1;
      if (this.caret != null) {
         this.caret.install(this);
         this.caret.addChangeListener(this.caretEvent);
      }

      ((JComponent)this).firePropertyChange("caret", var2, this.caret);
   }

   public Highlighter getHighlighter() {
      return this.highlighter;
   }

   public void setHighlighter(Highlighter var1) {
      if (this.highlighter != null) {
         this.highlighter.deinstall(this);
      }

      Highlighter var2 = this.highlighter;
      this.highlighter = var1;
      if (this.highlighter != null) {
         this.highlighter.install(this);
      }

      ((JComponent)this).firePropertyChange("highlighter", var2, var1);
   }

   public void setKeymap(Keymap var1) {
      Keymap var2 = this.keymap;
      this.keymap = var1;
      ((JComponent)this).firePropertyChange("keymap", var2, this.keymap);
      this.updateInputMap(var2, var1);
   }

   void updateInputMap(Keymap var1, Keymap var2) {
      InputMap var3 = ((JComponent)this).getInputMap(0);

      InputMap var4;
      for(var4 = var3; var3 != null && !(var3 instanceof KeymapWrapper); var3 = var3.getParent()) {
         var4 = var3;
      }

      if (var3 != null) {
         if (var2 == null) {
            if (var4 != var3) {
               var4.setParent(var3.getParent());
            } else {
               var4.setParent((InputMap)null);
            }
         } else {
            KeymapWrapper var5 = new KeymapWrapper(var2);
            var4.setParent(var5);
            if (var4 != var3) {
               ((InputMap)var5).setParent(var3.getParent());
            }
         }
      } else if (var2 != null) {
         var3 = ((JComponent)this).getInputMap(0);
         if (var3 != null) {
            KeymapWrapper var9 = new KeymapWrapper(var2);
            ((InputMap)var9).setParent(var3.getParent());
            var3.setParent(var9);
         }
      }

      ActionMap var10 = ((JComponent)this).getActionMap();

      ActionMap var6;
      for(var6 = var10; var10 != null && !(var10 instanceof KeymapActionMap); var10 = var10.getParent()) {
         var6 = var10;
      }

      if (var10 != null) {
         if (var2 == null) {
            if (var6 != var10) {
               var6.setParent(var10.getParent());
            } else {
               var6.setParent((ActionMap)null);
            }
         } else {
            KeymapActionMap var7 = new KeymapActionMap(var2);
            var6.setParent(var7);
            if (var6 != var10) {
               ((ActionMap)var7).setParent(var10.getParent());
            }
         }
      } else if (var2 != null) {
         var10 = ((JComponent)this).getActionMap();
         if (var10 != null) {
            KeymapActionMap var12 = new KeymapActionMap(var2);
            ((ActionMap)var12).setParent(var10.getParent());
            var10.setParent(var12);
         }
      }

   }

   public Keymap getKeymap() {
      return this.keymap;
   }

   public static Keymap addKeymap(String var0, Keymap var1) {
      DefaultKeymap var2 = new DefaultKeymap(var0, var1);
      if (var0 != null) {
         keymapTable.put(var0, var2);
      }

      return var2;
   }

   public static Keymap removeKeymap(String var0) {
      return (Keymap)keymapTable.remove(var0);
   }

   public static Keymap getKeymap(String var0) {
      return (Keymap)keymapTable.get(var0);
   }

   public static void loadKeymap(Keymap var0, KeyBinding[] var1, Action[] var2) {
      Hashtable var3 = new Hashtable();

      for(int var4 = 0; var4 < var2.length; ++var4) {
         Action var5 = var2[var4];
         String var6 = (String)var5.getValue("Name");
         var3.put(var6 != null ? var6 : "", var5);
      }

      for(int var7 = 0; var7 < var1.length; ++var7) {
         Action var8 = (Action)var3.get(var1[var7].actionName);
         if (var8 != null) {
            var0.addActionForKeyStroke(var1[var7].key, var8);
         }
      }

   }

   public Color getCaretColor() {
      return this.caretColor;
   }

   public void setCaretColor(Color var1) {
      Color var2 = this.caretColor;
      this.caretColor = var1;
      ((JComponent)this).firePropertyChange("caretColor", var2, this.caretColor);
   }

   public Color getSelectionColor() {
      return this.selectionColor;
   }

   public void setSelectionColor(Color var1) {
      Color var2 = this.selectionColor;
      this.selectionColor = var1;
      ((JComponent)this).firePropertyChange("selectionColor", var2, this.selectionColor);
   }

   public Color getSelectedTextColor() {
      return this.selectedTextColor;
   }

   public void setSelectedTextColor(Color var1) {
      Color var2 = this.selectedTextColor;
      this.selectedTextColor = var1;
      ((JComponent)this).firePropertyChange("selectedTextColor", var2, this.selectedTextColor);
   }

   public Color getDisabledTextColor() {
      return this.disabledTextColor;
   }

   public void setDisabledTextColor(Color var1) {
      Color var2 = this.disabledTextColor;
      this.disabledTextColor = var1;
      ((JComponent)this).firePropertyChange("disabledTextColor", var2, this.disabledTextColor);
   }

   public void replaceSelection(String var1) {
      Document var2 = this.getDocument();
      if (var2 != null) {
         try {
            int var3 = Math.min(this.caret.getDot(), this.caret.getMark());
            int var4 = Math.max(this.caret.getDot(), this.caret.getMark());
            if (var3 != var4) {
               var2.remove(var3, var4 - var3);
            }

            if (var1 != null && var1.length() > 0) {
               var2.insertString(var3, var1, (AttributeSet)null);
            }
         } catch (BadLocationException var5) {
            ((Component)this).getToolkit().beep();
         }
      }

   }

   public String getText(int var1, int var2) throws BadLocationException {
      return this.getDocument().getText(var1, var2);
   }

   public Rectangle modelToView(int var1) throws BadLocationException {
      return this.getUI().modelToView(this, var1);
   }

   public int viewToModel(Point var1) {
      return this.getUI().viewToModel(this, var1);
   }

   public void cut() {
      if (this.isEditable() && ((Component)this).isEnabled()) {
         try {
            Clipboard var1 = ((Component)this).getToolkit().getSystemClipboard();
            int var2 = Math.min(this.caret.getDot(), this.caret.getMark());
            int var3 = Math.max(this.caret.getDot(), this.caret.getMark());
            if (var2 != var3) {
               Document var4 = this.getDocument();
               String var5 = var4.getText(var2, var3 - var2);
               StringSelection var6 = new StringSelection(var5);
               var1.setContents(var6, defaultClipboardOwner);
               var4.remove(var2, var3 - var2);
            }
         } catch (BadLocationException var7) {
         }
      } else {
         ((Component)this).getToolkit().beep();
      }

   }

   public void copy() {
      try {
         Clipboard var1 = ((Component)this).getToolkit().getSystemClipboard();
         int var2 = Math.min(this.caret.getDot(), this.caret.getMark());
         int var3 = Math.max(this.caret.getDot(), this.caret.getMark());
         if (var2 != var3) {
            Document var4 = this.getDocument();
            String var5 = var4.getText(var2, var3 - var2);
            StringSelection var6 = new StringSelection(var5);
            var1.setContents(var6, defaultClipboardOwner);
         }
      } catch (BadLocationException var7) {
      }

   }

   public void paste() {
      Clipboard var1 = ((Component)this).getToolkit().getSystemClipboard();
      Transferable var2 = var1.getContents(this);
      if (this.isEditable() && ((Component)this).isEnabled() && var2 != null) {
         try {
            String var3 = (String)var2.getTransferData(DataFlavor.stringFlavor);
            this.replaceSelection(var3);
         } catch (Exception var4) {
            ((Component)this).getToolkit().beep();
         }
      } else {
         ((Component)this).getToolkit().beep();
      }

   }

   public void moveCaretPosition(int var1) {
      this.caret.moveDot(var1);
   }

   public void setFocusAccelerator(char var1) {
      var1 = Character.toUpperCase(var1);
      char var2 = this.focusAccelerator;
      this.focusAccelerator = var1;
      ((JComponent)this).firePropertyChange("focusAcceleratorKey", var2, this.focusAccelerator);
   }

   public char getFocusAccelerator() {
      return this.focusAccelerator;
   }

   public void read(Reader var1, Object var2) throws IOException {
      EditorKit var3 = this.getUI().getEditorKit(this);
      Document var4 = var3.createDefaultDocument();
      if (var2 != null) {
         var4.putProperty("stream", var2);
      }

      try {
         var3.read(var1, var4, 0);
         this.setDocument(var4);
      } catch (BadLocationException var6) {
         throw new IOException(((Throwable)var6).getMessage());
      }
   }

   public void write(Writer var1) throws IOException {
      Document var2 = this.getDocument();

      try {
         this.getUI().getEditorKit(this).write(var1, var2, 0, var2.getLength());
      } catch (BadLocationException var4) {
         throw new IOException(((Throwable)var4).getMessage());
      }
   }

   public void removeNotify() {
      super.removeNotify();
      if (focusedComponent == this) {
         focusedComponent = null;
      }

   }

   public boolean isFocusTraversable() {
      return ((Component)this).isEnabled();
   }

   public void setCaretPosition(int var1) {
      Document var2 = this.getDocument();
      if (var2 != null) {
         if (var1 > var2.getLength() || var1 < 0) {
            throw new IllegalArgumentException("bad position: " + var1);
         }

         this.caret.setDot(var1);
      }

   }

   public int getCaretPosition() {
      return this.caret.getDot();
   }

   public void setText(String var1) {
      try {
         Document var2 = this.getDocument();
         var2.remove(0, var2.getLength());
         var2.insertString(0, var1, (AttributeSet)null);
      } catch (BadLocationException var3) {
         ((Component)this).getToolkit().beep();
      }

   }

   public String getText() {
      Document var1 = this.getDocument();

      String var2;
      try {
         var2 = var1.getText(0, var1.getLength());
      } catch (BadLocationException var4) {
         var2 = null;
      }

      return var2;
   }

   public String getSelectedText() {
      String var1 = null;
      int var2 = Math.min(this.caret.getDot(), this.caret.getMark());
      int var3 = Math.max(this.caret.getDot(), this.caret.getMark());
      if (var2 != var3) {
         try {
            Document var4 = this.getDocument();
            var1 = var4.getText(var2, var3 - var2);
         } catch (BadLocationException var5) {
            throw new IllegalArgumentException(((Throwable)var5).getMessage());
         }
      }

      return var1;
   }

   public boolean isEditable() {
      return this.editable;
   }

   public void setEditable(boolean var1) {
      if (var1 != this.editable) {
         boolean var2 = this.editable;
         this.editable = var1;
         if (this.editable) {
            ((Component)this).setCursor(Cursor.getPredefinedCursor(2));
         } else {
            ((Component)this).setCursor(Cursor.getPredefinedCursor(0));
         }

         ((JComponent)this).firePropertyChange("editable", new Boolean(var2), new Boolean(this.editable));
         ((Component)this).repaint();
      }

   }

   public int getSelectionStart() {
      int var1 = Math.min(this.caret.getDot(), this.caret.getMark());
      return var1;
   }

   public void setSelectionStart(int var1) {
      this.select(var1, this.getSelectionEnd());
   }

   public int getSelectionEnd() {
      int var1 = Math.max(this.caret.getDot(), this.caret.getMark());
      return var1;
   }

   public void setSelectionEnd(int var1) {
      this.select(this.getSelectionStart(), var1);
   }

   public void select(int var1, int var2) {
      if (var1 < 0) {
         var1 = 0;
      }

      if (var2 > this.getDocument().getLength()) {
         var2 = this.getDocument().getLength();
      }

      if (var2 < var1) {
         var2 = var1;
      }

      if (var1 > var2) {
         var1 = var2;
      }

      this.setCaretPosition(var1);
      this.moveCaretPosition(var2);
   }

   public void selectAll() {
      Document var1 = this.getDocument();
      if (var1 != null) {
         this.setCaretPosition(0);
         this.moveCaretPosition(var1.getLength());
      }

   }

   public Dimension getPreferredScrollableViewportSize() {
      return ((JComponent)this).getPreferredSize();
   }

   public int getScrollableUnitIncrement(Rectangle var1, int var2, int var3) {
      switch (var2) {
         case 0:
            return var1.width / 10;
         case 1:
            return var1.height / 10;
         default:
            throw new IllegalArgumentException("Invalid orientation: " + var2);
      }
   }

   public int getScrollableBlockIncrement(Rectangle var1, int var2, int var3) {
      switch (var2) {
         case 0:
            return var1.width;
         case 1:
            return var1.height;
         default:
            throw new IllegalArgumentException("Invalid orientation: " + var2);
      }
   }

   public boolean getScrollableTracksViewportWidth() {
      if (((Component)this).getParent() instanceof JViewport) {
         return ((JViewport)((Component)this).getParent()).getWidth() > ((JComponent)this).getPreferredSize().width;
      } else {
         return false;
      }
   }

   public boolean getScrollableTracksViewportHeight() {
      if (((Component)this).getParent() instanceof JViewport) {
         return ((JViewport)((Component)this).getParent()).getHeight() > ((JComponent)this).getPreferredSize().height;
      } else {
         return false;
      }
   }

   public AccessibleContext getAccessibleContext() {
      if (super.accessibleContext == null) {
         super.accessibleContext = new AccessibleJTextComponent(this);
      }

      return super.accessibleContext;
   }

   private void readObject(ObjectInputStream var1) throws IOException, ClassNotFoundException {
      var1.defaultReadObject();
      this.caretEvent = new MutableCaretEvent(this);
      ((Component)this).addMouseListener(this.caretEvent);
      ((Component)this).addFocusListener(this.caretEvent);
   }

   protected String paramString() {
      String var1 = this.editable ? "true" : "false";
      String var2 = this.caretColor != null ? this.caretColor.toString() : "";
      String var3 = this.selectionColor != null ? this.selectionColor.toString() : "";
      String var4 = this.selectedTextColor != null ? this.selectedTextColor.toString() : "";
      String var5 = this.disabledTextColor != null ? this.disabledTextColor.toString() : "";
      String var6 = this.margin != null ? this.margin.toString() : "";
      return super.paramString() + ",caretColor=" + var2 + ",disabledTextColor=" + var5 + ",editable=" + var1 + ",margin=" + var6 + ",selectedTextColor=" + var4 + ",selectionColor=" + var3;
   }

   static final JTextComponent getFocusedComponent() {
      return focusedComponent;
   }

   protected void processInputMethodEvent(InputMethodEvent var1) {
      super.processInputMethodEvent(var1);
      if (!var1.isConsumed()) {
         if (!this.isEditable()) {
            return;
         }

         switch (((AWTEvent)var1).getID()) {
            case 1100:
               this.replaceInputMethodText(var1);
            case 1101:
               this.setInputMethodCaretPosition(var1);
            default:
               var1.consume();
         }
      }

   }

   public InputMethodRequests getInputMethodRequests() {
      if (this.inputMethodRequestsHandler == null) {
         this.inputMethodRequestsHandler = new InputMethodRequestsHandler(this);
      }

      return this.inputMethodRequestsHandler;
   }

   public void addInputMethodListener(InputMethodListener var1) {
      super.addInputMethodListener(var1);
      if (var1 != null) {
         this.needToSendKeyTypedEvent = false;
      }

   }

   private void replaceInputMethodText(InputMethodEvent var1) {
      int var2 = var1.getCommittedCharacterCount();
      AttributedCharacterIterator var3 = var1.getText();
      Document var5 = this.getDocument();
      if (this.composedTextStart != null) {
         try {
            int var6 = this.composedTextStart.getOffset();
            var5.remove(var6, this.composedTextEnd.getOffset() - var6);
         } catch (BadLocationException var9) {
         }

         this.composedTextStart = this.composedTextEnd = null;
         this.composedText = null;
         this.composedTextContent = null;
      }

      if (var3 != null) {
         var3.first();
         if (var2 > 0) {
            if (this.needToSendKeyTypedEvent) {
               for(char var11 = var3.current(); var2 > 0; --var2) {
                  KeyEvent var13 = new KeyEvent(this, 400, System.currentTimeMillis(), 0, 0, var11);
                  ((JComponent)this).processKeyEvent(var13);
                  var11 = var3.next();
               }
            } else {
               StringBuffer var10 = new StringBuffer();

               for(char var7 = var3.current(); var2 > 0; --var2) {
                  var10.append(var7);
                  var7 = var3.next();
               }

               this.mapCommittedTextToAction(new String(var10));
            }
         }

         int var4 = var3.getIndex();
         if (var4 < var3.getEndIndex()) {
            this.createComposedString(var4, var3);
            SimpleAttributeSet var12 = new SimpleAttributeSet();
            var12.addAttribute(StyleConstants.ComposedTextAttribute, this.composedText);

            try {
               this.replaceSelection((String)null);
               var5.insertString(this.caret.getDot(), this.composedTextContent, var12);
               this.composedTextStart = var5.createPosition(this.caret.getDot() - this.composedTextContent.length());
               this.composedTextEnd = var5.createPosition(this.caret.getDot());
            } catch (BadLocationException var8) {
               this.composedTextStart = this.composedTextEnd = null;
               this.composedText = null;
               this.composedTextContent = null;
            }
         }
      }

   }

   private void createComposedString(int var1, AttributedCharacterIterator var2) {
      Document var3 = this.getDocument();
      StringBuffer var4 = new StringBuffer();

      for(char var5 = var2.setIndex(var1); var5 != '\uffff'; var5 = var2.next()) {
         var4.append(var5);
      }

      this.composedTextContent = new String(var4);
      this.composedText = new AttributedString(var2, var1, var2.getEndIndex());
   }

   private void mapCommittedTextToAction(String var1) {
      Keymap var2 = this.getKeymap();
      if (var2 != null) {
         Action var3 = null;
         if (var1.length() == 1) {
            KeyStroke var4 = KeyStroke.getKeyStroke(var1.charAt(0));
            var3 = var2.getAction(var4);
         }

         if (var3 == null) {
            var3 = var2.getDefaultAction();
         }

         if (var3 != null) {
            ActionEvent var5 = new ActionEvent(this, 1001, var1);
            var3.actionPerformed(var5);
         }
      }

   }

   private void setInputMethodCaretPosition(InputMethodEvent var1) {
      if (this.composedTextStart != null) {
         int var2 = this.composedTextStart.getOffset();
         if (!(this.caret instanceof ComposedTextCaret)) {
            if (this.composedTextCaret == null) {
               this.composedTextCaret = new ComposedTextCaret(this);
            }

            this.originalCaret = this.caret;
            this.exchangeCaret(this.originalCaret, this.composedTextCaret);
         }

         TextHitInfo var3 = var1.getCaret();
         if (var3 != null) {
            var2 += var3.getInsertionIndex();
         }

         this.caret.setDot(var2);
      } else if (this.caret instanceof ComposedTextCaret) {
         int var4 = this.caret.getDot();
         this.exchangeCaret(this.caret, this.originalCaret);
         this.caret.setDot(var4);
      }

   }

   private void exchangeCaret(Caret var1, Caret var2) {
      int var3 = var1.getBlinkRate();
      this.setCaret(var2);
      this.caret.setBlinkRate(var3);
      this.caret.setVisible(((JComponent)this).hasFocus());
   }

   private boolean isProcessInputMethodEventOverridden() {
      Boolean var1 = (Boolean)AccessController.doPrivileged(new 4(this));
      return var1;
   }

   // $FF: synthetic method
   static Class class$(String var0) {
      try {
         return Class.forName(var0);
      } catch (ClassNotFoundException var2) {
         throw new NoClassDefFoundError(((Throwable)var2).getMessage());
      }
   }

   // $FF: synthetic method
   static Document access$000(JTextComponent var0) {
      return var0.model;
   }

   // $FF: synthetic method
   static JTextComponent access$102(JTextComponent var0) {
      focusedComponent = var0;
      return var0;
   }

   // $FF: synthetic method
   static AttributedString access$200(JTextComponent var0) {
      return var0.composedText;
   }

   // $FF: synthetic method
   static Position access$300(JTextComponent var0) {
      return var0.composedTextStart;
   }

   // $FF: synthetic method
   static Position access$400(JTextComponent var0) {
      return var0.composedTextEnd;
   }

   static {
      try {
         keymapTable = new Hashtable(17);
         Keymap var0 = addKeymap("default", (Keymap)null);
         var0.setDefaultAction(new DefaultEditorKit.DefaultKeyTypedAction());
      } catch (Throwable var1) {
         var1.printStackTrace();
      }

   }
}
